/*
 *
 * CLEX File Manager
 *
 * Copyright (C) 1997-2000 Tempest s.r.o. http://www.tempest.sk
 * Copyright (C) 2001-2002 Vlado Potisk <vlado_potisk@clex.sk>
 *
 * CLEX is free software without warranty of any kind; see the
 * GNU General Public License as set out in the "COPYING" document
 * which accompanies the CLEX File Manager package.
 *
 * CLEX can be downloaded from http://www.clex.sk
 *
 */

#include <autoconf.h>

#include <sys/types.h>	/* clex.h */
#include <stdio.h>		/* putchar() */
#include <signal.h>		/* sigaction() */
#include <stdarg.h>		/* va_list */
#include <stdlib.h>		/* exit() */

#include "clex.h"
#include "err.h"

#include "inout.h"		/* curses_stop() */
#include "tty.h"		/* tty_reset() */
#include "xterm_title.h"/* xterm_title_restore() */

static RETSIGTYPE
handler(int signo)
{
	err_exit("Signal %s caught",
	  signo == SIGTERM ? "SIGTERM" : "SIGHUP");
}

void
signal_initialize(void)
{
	struct sigaction act;

	/* ignore keyboard generated signals */
	act.sa_handler = SIG_IGN;
	sigemptyset(&act.sa_mask);
	sigaction(SIGINT,&act,0);
	sigaction(SIGQUIT,&act,0);

	/* catch usual termination signals */
	act.sa_handler = handler;
	act.sa_flags = 0;
	sigaddset(&act.sa_mask,SIGTERM);
	sigaddset(&act.sa_mask,SIGHUP);
	sigaction(SIGTERM,&act,0);
	sigaction(SIGHUP,&act,0);
}

/*
 * err_exit() is the only exit function that terminates CLEX main
 * process. It is used for normal (no error) termination as well.
 */
void
err_exit(const char *format, ...)
{
	va_list argptr;

	/* these cleanup functions below must not call err_exit() */
	xterm_title_restore();
	if (display.curses)
		curses_stop();
	tty_reset();

	fputs("\nTerminating CLEX: ",stdout);
	va_start(argptr,format);
	vprintf(format,argptr);
	va_end(argptr);
	putchar('\n');
	exit(0);
	/* NOTREACHED */
}
