/*
 *
 * CLEX File Manager
 *
 * Copyright (C) 1997-2000 Tempest s.r.o. http://www.tempest.sk
 * Copyright (C) 2001-2002 Vlado Potisk <vlado_potisk@clex.sk>
 *
 * CLEX is free software without warranty of any kind; see the
 * GNU General Public License as set out in the "COPYING" document
 * which accompanies the CLEX File Manager package.
 *
 * CLEX can be downloaded from http://www.clex.sk
 *
 */

#include <autoconf.h>

#include <sys/types.h>		/* clex.h */
#include <string.h>			/* strcmp() */

#include "clex.h"
#include "history.h"

#include "cfg.h"			/* config_num() */
#include "edit.h"			/* edit_update() */
#include "exec.h"			/* execute_cmd() */
#include "inout.h"			/* win_remark() */
#include "panel.h"			/* pan_adjust() */
#include "util.h"			/* emalloc() */
#include "ustring.h"		/* us_copy() */

static USTRING **history = 0;	/* command history list */
static int history_cnt;			/* 'history' element counter */
static int cfg_hist;			/* number of allocated entries */
static int pn_index;			/* index for previous/next cmd */
static USTRING save_line = { 0,0 };		/* for saving command line */

void
hist_reconfig(void)
{
	if (history)
		us_free_array(history,cfg_hist);
	history_cnt = 0;
	pn_index = -1;
	cfg_hist = config_num(CFG_HISTORY);
	panel_hist.list = history = us_alloc_array(cfg_hist);
}

void
hist_prepare(void)
{
	panel_hist.top = 0;
	panel_hist.cnt = history_cnt;
	if (pn_index) {
		panel_hist.curs = pn_index;
		pan_adjust((PANEL *)&panel_hist);
	}
	else 
		panel_hist.curs = 0;

	panel = (PANEL *)&panel_hist;
	textline = &line_cmd;
}

void
hist_reset_index(void)
{
	pn_index = -1;
}

/*
 * hist_save() pushes the command 'cmd' on the top
 * of the command history list.
 */
void
hist_save(const char *cmd)
{
	int i;
	FLAG new;
	USTRING *x, *top;

	hist_reset_index();

	new = 1;
	for (top = history[0], i = 0; i < cfg_hist; i++) {
		x = history[i];
		history[i] = top;
		top = x;
		if (i == history_cnt) {
			history_cnt++;
			break;
		}
		if (strcmp(PUSTR(top),cmd) == 0) {
			/* avoid duplicates */
			new = 0;
			break;
		}
	}
	if (new)
		us_copy(top,cmd);
	history[0] = top;
}

/* copy next (i.e. more recent) command from the history list */
void
cx_hist_next(void)
{
	if (pn_index == -1) {
		win_remark("top of history list");
		return;
	}

	if (pn_index-- == 0)
		edit_putstr(USTR(save_line));
	else
		edit_putstr(PUSTR(history[pn_index]));
}

/* copy previous (i.e. older) command from the history list */
void
cx_hist_prev(void)
{
	if (pn_index >= history_cnt - 1) {
		win_remark("bottom of history list");
		return;
	}

	if (++pn_index == 0)
		us_copy(&save_line,USTR(line_cmd.line));
	edit_putstr(PUSTR(history[pn_index]));
}

void
cx_hist_complete(void)
{
	int i;

	for (i = 0; i < history_cnt; i++)
		if (strncmp(PUSTR(history[i]),USTR(line_cmd.line),
		  line_cmd.size) == 0) {
			textline->curs = textline->size;
			edit_insertstr(PUSTR(history[i]) + textline->size,0);
			return;
		}
	win_remark("cannot complete this command (no match found)");
}

void
cx_hist_paste(void)
{
	if (panel_hist.cnt && panel_hist.curs >= 0)
		edit_insertstr(PUSTR(history[panel_hist.curs]),0);
}

void
cx_hist_enter(void)
{
	char *cmd;

	next_mode = MODE_SPECIAL_RETURN;
	if (line_cmd.size)
		cmd = USTR(line_cmd.line);
	else if (panel_hist.cnt && panel_hist.curs >= 0)
		cmd = PUSTR(history[panel_hist.curs]);
	else
		return;

	/*
	 * Right after the command execution CLEX will return
	 * from the history mode back to the file mode. This
	 * little hack prevents one needless screen redrawing.
	 */
	panel = (PANEL *)ppanel_file;

	if (execute_cmd(cmd))
		cx_edit_kill();
}
