/*
 *
 * CLEX File Manager
 *
 * Copyright (C) 1997-2000 Tempest s.r.o. http://www.tempest.sk
 * Copyright (C) 2001-2002 Vlado Potisk <vlado_potisk@clex.sk>
 *
 * CLEX is free software without warranty of any kind; see the
 * GNU General Public License as set out in the "COPYING" document
 * which accompanies the CLEX File Manager package.
 *
 * CLEX can be downloaded from http://www.clex.sk
 *
 */

#include <autoconf.h>

#include <sys/types.h>	/* clex.h */
#include <stdlib.h>		/* free() */
#include <string.h>		/* strncmp() */

#include "clex.h"
#include "locate.h"

#include "cfg.h"		/* config_num() */
#include "edit.h"		/* edit_setprompt() */
#include "panel.h"		/* pan_adjust() */
#include "inout.h"		/* win_panel() */
#include "ustring.h"	/* USTRING() */
#include "util.h"		/* emalloc() */

static FLAG dotwarning;

void
locate_prepare(void)
{
	int i, cnt;

	dotwarning = config_num(CFG_SHOW_HIDDEN) == 0;

	panel_locate.top = ppanel_file->top;
	panel_locate.curs = ppanel_file->curs;
	cnt = panel_locate.cnt = ppanel_file->cnt;
	if (panel_locate.fe_slots < cnt) {
		if (panel_locate.files)
			free(panel_locate.files);
		panel_locate.files = emalloc(cnt * sizeof(FILE_ENTRY *));
	}
	for (i = 0; i < cnt; i++)
		panel_locate.files[i] = ppanel_file->files[i];
	panel = (PANEL *)&panel_locate;

	/* visually different prompt */
	edit_setprompt(&line_tmp,"==> Locate file: ");
	textline = &line_tmp;
	edit_nu_kill();
}

void
locate_update(void)
{
	int i, j, len;
	const char *str;
	FILE_ENTRY *pfe, *curspfe;

	if (ppanel_file->cnt == 0)
		return;

	str = USTR(textline->line);
	len = textline->size;
	if (*str == '.' && TCLR(dotwarning))
		win_warning("LOCATE: SHOW_HIDDEN (show hidden .files) "
		  "is not enabled.");
	curspfe = panel_locate.files[panel_locate.curs];
	for (i = j = 0; i < ppanel_file->cnt; i++) {
		pfe = ppanel_file->files[i];
		if (pfe == curspfe)
			panel_locate.curs = j;
		if (len == 0 || strncmp(str,USTR(pfe->file),len) == 0)
			panel_locate.files[j++] = pfe;
	}
	panel_locate.cnt = j;

	pan_adjust(panel);
	win_panel();
}

/* leave the locate mode, cursor is set to the selected file name */
void
cx_locate_go(void)
{
	int i;
	FILE_ENTRY *pfe;

	if (panel_locate.cnt) {
		pfe = panel_locate.files[panel_locate.curs];
		for (i = 0 ; i < ppanel_file->cnt; i++)
			if (ppanel_file->files[i] == pfe) {
				ppanel_file->curs = i;
				pan_adjust((PANEL *)ppanel_file);
				break;
			}
	}
	next_mode = MODE_SPECIAL_RETURN;
}

/* tab = insert selected filename to the command line */
void
cx_locate_tab(void)
{
	if (panel_locate.cnt == 0)
		return;

	if (kbd_esc())
		cx_edit_fullpath();
	else
		cx_edit_filename();
}
