/*
 *
 * CLEX File Manager
 *
 * Copyright (C) 1997-2000 Tempest s.r.o. http://www.tempest.sk
 * Copyright (C) 2001-2002 Vlado Potisk <vlado_potisk@clex.sk>
 *
 * CLEX is free software without warranty of any kind; see the
 * GNU General Public License as set out in the "COPYING" document
 * which accompanies the CLEX File Manager package.
 *
 * CLEX can be downloaded from http://www.clex.sk
 *
 */

#include <autoconf.h>

#include <sys/types.h>	/* clex.h */
#include <stdio.h>		/* puts() */
#include <stdlib.h>		/* putenv() */
#include <string.h>		/* strcmp() */

#include "clex.h"

#include "cfg.h"		/* config_initialize() */
#include "completion.h"	/* completion_initialize() */
#include "control.h"	/* control_loop() */
#include "directory.h"	/* dir_initialize() */
#include "err.h"		/* signal_initialize() */
#include "exec.h"		/* exec_initialize() */
#include "filepanel.h"	/* files_initialize() */
#include "help.h"		/* help_initialize() */
#include "history.h"	/* hist_initialize() */
#include "inout.h"		/* curses_initialize() */
#include "list.h"		/* list_initialize() */
#include "select.h"		/* select_initialize() */
#include "sort.h"		/* sort_initialize() */
#include "tty.h"		/* tty_initialize() */
#include "userdata.h"	/* userdata_initialize() */
#include "ustring.h"	/* US_INIT() */
#include "util.h"		/* base_name() */
#include "xterm_title.h"/* xterm_title_initialize() */

/* GLOBAL VARIABLES */
PANEL *panel = 0;
PANEL_COMPLETION panel_compl = { 0,0,0,-1,PANEL_TYPE_COMPL };
PANEL_CONFIG panel_config = { 0,0,0,-1,PANEL_TYPE_CONFIG };
PANEL_DIR_BROWSE panel_dir_browse = { 0,0,0,-1,PANEL_TYPE_DIR_BROWSE };
PANEL_DIR_SPLIT panel_dir_split = { 0,0,0,-1,PANEL_TYPE_DIR_SPLIT };
PANEL_FILE *ppanel_file;
PANEL_HELP panel_help = { 0,0,0,0,PANEL_TYPE_HELP };
PANEL_HISTORY panel_hist = { 0,0,0,-1,PANEL_TYPE_HIST };
PANEL_LOCATE panel_locate = { 0,0,0,0,PANEL_TYPE_LOCATE,0,0 };
PANEL_MENU panel_menu = { 0,0,0,-1,PANEL_TYPE_MENU };
PANEL_MENU panel_config_confirm = { 0,0,0,0,PANEL_TYPE_MENU };
PANEL_SORT panel_sort = { 0,0,0,-1,PANEL_TYPE_SORT };
DISPLAY display = { 0,0,0,0,0 };
CLEX_USER clex_user;
TEXTLINE *textline = 0, line_cmd, line_tmp, line_dir;
CODE next_mode;
FLAG adminmode = 0;

/* start CLEX in ten easy steps */
int
main(int argc, char *argv[])
{
	signal_initialize();	/* #1 - before anything that might fail */
	tty_initialize();	/* #2 - before any I/O */

	/* #3 - say hello */
	puts("\n\n\n\nStarting CLEX " VERSION);
	fflush(stdout);

	/* #4 - check command line arguments */
	if (argc > 1) {
		if (argc == 2 && strcmp(argv[1],"-a") == 0)
			adminmode = 1;	/* config in admin mode */
		else
			err_exit("Usage: %s [-a]\n",base_name(argv[0]));
	}

	/* #5 - initialize miscellaneous variables */
#ifdef HAVE_PUTENV
	putenv("CLEX=" VERSION);/* let's have $CLEX (just so) */
#endif
	US_INIT(line_tmp.line);	/* belongs to nowhere, initialize it here */

	userdata_initialize();	/* #6 - before config */
	config_initialize();	/* #7 - read configuration */

	/* #8 - initialize the rest, the order is no longer important */
	help_initialize();
	if (!adminmode) {
		/* all these features are not accessed in admin mode */
		completion_initialize();
		dir_initialize();
		exec_initialize();
		files_initialize();
		hist_initialize();
		list_initialize();
		menu_initialize();
		select_initialize();
		sort_initialize();
		xterm_title_initialize();
	}

	/* #9 - start CURSES screen handling */
	curses_initialize();

	/* #10 - let's go ! */
	cx_version();
	control_loop(adminmode ? MODE_CONFIG_MAIN : MODE_FILE_MAIN);

	/* NOTREACHED */
	return 0;
}
